@@ -162,13 +162,15 @@ dates_difference(N_int year1, N_int mm1, N_int dd1,
 
 /*** END OF lib_date routines ********************************************/
 
+/* Spacing around day/week headers and main area, inside those windows */
 #define CALENDAR_MARGIN		 0
-#define CALENDAR_YSEP		 4
-#define CALENDAR_XSEP		 4
+/* Spacing around day/week headers and main area, outside those windows */
 #define INNER_BORDER		 4
+/* Separation between day headers and main area */
+#define CALENDAR_YSEP		 4
+/* Separation between week headers and main area */
+#define CALENDAR_XSEP		 4
 
-#define DAY_XPAD		 2
-#define DAY_YPAD		 2
 #define DAY_XSEP		 0 /* not really good for small calendar */
 #define DAY_YSEP		 0 /* not really good for small calendar */
 
@@ -258,7 +260,7 @@ typedef void (*GtkCalendarSignalDate) (GtkObject *object, guint arg1, guint arg2
 
 static void gtk_calendar_class_init	(GtkCalendarClass *class);
 static void gtk_calendar_init		(GtkCalendar *calendar);
-static void gtk_calendar_destroy	(GtkObject *calendar);
+static void gtk_calendar_finalize	(GObject *calendar);
 static void gtk_calendar_realize	(GtkWidget *widget);
 static void gtk_calendar_unrealize	(GtkWidget *widget);
 static void gtk_calendar_size_request	(GtkWidget *widget,
@@ -334,15 +336,17 @@ gtk_calendar_get_type (void)
 static void
 gtk_calendar_class_init (GtkCalendarClass *class)
 {
+  GObjectClass   *gobject_class;
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
-  
+
+  gobject_class = (GObjectClass*)  class;
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   
   parent_class = gtk_type_class (GTK_TYPE_WIDGET);
   
-  object_class->destroy = gtk_calendar_destroy;
+  gobject_class->finalize = gtk_calendar_finalize;
 
   widget_class->realize = gtk_calendar_realize;
   widget_class->unrealize = gtk_calendar_unrealize;
@@ -559,7 +563,7 @@ left_x_for_column (GtkCalendar *calendar,
 
   width = GTK_CALENDAR_PRIVATE_DATA (calendar)->day_width;
   if (calendar->display_flags & GTK_CALENDAR_SHOW_WEEK_NUMBERS)
-    x_left =  DAY_XSEP + (width + DAY_XSEP) * column;
+    x_left = CALENDAR_XSEP + (width + DAY_XSEP) * column;
   else
     x_left = CALENDAR_MARGIN + (width + DAY_XSEP) * column;
   
@@ -853,12 +857,12 @@ gtk_calendar_realize_arrows (GtkWidget *widget)
 			      + private_data->max_month_width);
 	      break;
 	    case ARROW_YEAR_LEFT:
-	      attributes.x = (widget->allocation.width - 4
+	      attributes.x = (widget->allocation.width - 2 * widget->style->xthickness
 			      - (3 + 2*private_data->arrow_width 
 				 + private_data->max_year_width));
 	      break;
 	    case ARROW_YEAR_RIGHT:
-	      attributes.x = (widget->allocation.width - 4 
+	      attributes.x = (widget->allocation.width - 2 * widget->style->xthickness 
 			      - 3 - private_data->arrow_width);
 	      break;
 	    }
@@ -903,8 +907,8 @@ gtk_calendar_realize_header (GtkWidget *widget)
       attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
       attributes.x = widget->style->xthickness;
       attributes.y = widget->style->ythickness;
-      attributes.width = widget->allocation.width - 4;
-      attributes.height = private_data->header_h;
+      attributes.width = widget->allocation.width - 2 * attributes.x;
+      attributes.height = private_data->header_h - 2 * attributes.y;
       private_data->header_win = gdk_window_new (widget->window,
 					     &attributes, attributes_mask);
       
@@ -1014,8 +1018,6 @@ gtk_calendar_realize (GtkWidget *widget)
   gint attributes_mask;
   GdkGCValues values;
 
-  g_return_if_fail (GTK_IS_CALENDAR (widget));
-  
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
   
@@ -1050,11 +1052,7 @@ gtk_calendar_realize (GtkWidget *widget)
 			    | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
 			    | GDK_POINTER_MOTION_MASK | GDK_LEAVE_NOTIFY_MASK);
   
-  if (calendar->display_flags & GTK_CALENDAR_SHOW_WEEK_NUMBERS)
-    attributes.x = private_data->week_width;
-  else
-    attributes.x = 0;
-  attributes.x += (widget->style->xthickness + INNER_BORDER);
+  attributes.x = private_data->week_width + (widget->style->ythickness + INNER_BORDER);
   attributes.y = (private_data->header_h + private_data->day_name_h 
 		  + (widget->style->ythickness + INNER_BORDER));
   attributes.width = (widget->allocation.width - attributes.x 
@@ -1088,8 +1086,6 @@ gtk_calendar_unrealize (GtkWidget *widget)
   GtkCalendarPrivateData *private_data;
   gint i;
   
-  g_return_if_fail (GTK_IS_CALENDAR (widget));
-  
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
   
@@ -1152,9 +1148,15 @@ gtk_calendar_size_request (GtkWidget	  *widget,
   gint calendar_margin = CALENDAR_MARGIN;
   gint header_width, main_width;
   gint max_header_height = 0;
+  gint focus_width;
+  gint focus_padding;
   
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
+  gtk_widget_style_get (GTK_WIDGET (widget),
+			"focus-line-width", &focus_width,
+			"focus-padding", &focus_padding,
+			NULL);
 
   layout = gtk_widget_create_pango_layout (widget, NULL);
   
@@ -1205,6 +1207,9 @@ gtk_calendar_size_request (GtkWidget	  *widget,
   
   private_data->max_day_char_width = 0;
   private_data->min_day_width = 0;
+  private_data->max_label_char_ascent = 0;
+  private_data->max_label_char_descent = 0;
+
   for (i = 0; i < 9; i++)
     {
       sprintf (buffer, "%d%d", i, i);
@@ -1245,13 +1250,13 @@ gtk_calendar_size_request (GtkWidget	  *widget,
 						 logical_rect.width / 2);
       }
   
-  main_width = (7 * (private_data->min_day_width + DAY_XPAD * 2) + (DAY_XSEP * 6) + CALENDAR_MARGIN * 2
+  main_width = (7 * (private_data->min_day_width + (focus_padding + focus_width) * 2) + (DAY_XSEP * 6) + CALENDAR_MARGIN * 2
 		+ (private_data->max_week_char_width
-		   ? private_data->max_week_char_width * 2 + DAY_XPAD * 2 + CALENDAR_XSEP * 2
+		   ? private_data->max_week_char_width * 2 + (focus_padding + focus_width) * 2 + CALENDAR_XSEP * 2
 		   : 0));
   
   
-  requisition->width = MAX (header_width+4, main_width + (widget->style->xthickness + INNER_BORDER) *2);
+  requisition->width = MAX (header_width, main_width + INNER_BORDER * 2) + widget->style->xthickness * 2;
   
   /*
    * Calculate the requisition height for the widget.
@@ -1270,7 +1275,7 @@ gtk_calendar_size_request (GtkWidget	  *widget,
     {
       private_data->day_name_h = (private_data->max_label_char_ascent
 				  + private_data->max_label_char_descent
-				  + 2 * DAY_YPAD + calendar_margin);
+				  + 2 * (focus_padding + focus_width) + calendar_margin);
       calendar_margin = CALENDAR_YSEP;
     } 
   else
@@ -1281,23 +1286,9 @@ gtk_calendar_size_request (GtkWidget	  *widget,
   private_data->main_h = (CALENDAR_MARGIN + calendar_margin
 			  + 6 * (private_data->max_day_char_ascent
 				 + private_data->max_day_char_descent 
-				 + DAY_YPAD * 2)
+				 + 2 * (focus_padding + focus_width))
 			  + DAY_YSEP * 5);
   
-  /* 
-   * If we display weeknumbers we need some extra space 
-   */
-  
-  if (calendar->display_flags & GTK_CALENDAR_SHOW_WEEK_NUMBERS)
-    {
-      private_data->main_h = MAX (private_data->main_h,
-				  (CALENDAR_MARGIN + calendar_margin
-				   + 6 * (private_data->max_day_char_ascent 
-					  + private_data->max_day_char_descent 
-					  + DAY_YPAD * 2)
-				   + DAY_YSEP * 5));
-    }
-  
   height = (private_data->header_h + private_data->day_name_h 
 	    + private_data->main_h);
   
@@ -1312,9 +1303,8 @@ gtk_calendar_size_allocate (GtkWidget	  *widget,
 {
   GtkCalendar *calendar;
   GtkCalendarPrivateData *private_data;
-  
-  g_return_if_fail (GTK_IS_CALENDAR (widget));
-  g_return_if_fail (allocation != NULL);
+  gint xthickness = widget->style->xthickness;
+  gint ythickness = widget->style->xthickness;
   
   widget->allocation = *allocation;
   
@@ -1324,19 +1314,19 @@ gtk_calendar_size_allocate (GtkWidget	  *widget,
   if (calendar->display_flags & GTK_CALENDAR_SHOW_WEEK_NUMBERS)
     {
       private_data->day_width = (private_data->min_day_width
-			     * ((allocation->width - (widget->style->xthickness + INNER_BORDER) * 2
-				 - (CALENDAR_MARGIN * 2) -  (DAY_XSEP * 7) - CALENDAR_XSEP * 2))
+			     * ((allocation->width - (xthickness + INNER_BORDER) * 2
+				 - (CALENDAR_MARGIN * 2) -  (DAY_XSEP * 6) - CALENDAR_XSEP * 2))
 			     / (7 * private_data->min_day_width + private_data->max_week_char_width * 2));
-      private_data->week_width = ((allocation->width - (widget->style->xthickness + INNER_BORDER) * 2
-			       - (CALENDAR_MARGIN * 2) - (DAY_XSEP * 7) - CALENDAR_XSEP * 2 )
+      private_data->week_width = ((allocation->width - (xthickness + INNER_BORDER) * 2
+			       - (CALENDAR_MARGIN * 2) - (DAY_XSEP * 6) - CALENDAR_XSEP * 2 )
 			      - private_data->day_width * 7 + CALENDAR_MARGIN + CALENDAR_XSEP);
     } 
   else 
     {
       private_data->day_width = (allocation->width
-			     - (widget->style->xthickness + INNER_BORDER) * 2
+			     - (xthickness + INNER_BORDER) * 2
 			     - (CALENDAR_MARGIN * 2)
-			     - (DAY_XSEP * 7))/7;
+			     - (DAY_XSEP * 6))/7;
       private_data->week_width = 0;
     }
   
@@ -1347,8 +1337,8 @@ gtk_calendar_size_allocate (GtkWidget	  *widget,
 			      allocation->width, allocation->height);
       if (private_data->header_win)
 	gdk_window_move_resize (private_data->header_win,
-				2, 2,
-				allocation->width-4, private_data->header_h);
+				xthickness, ythickness,
+				allocation->width - 2 * xthickness, private_data->header_h);
       if (private_data->arrow_win[ARROW_MONTH_LEFT])
 	gdk_window_move_resize (private_data->arrow_win[ARROW_MONTH_LEFT],
 				3, 3,
@@ -1363,7 +1353,7 @@ gtk_calendar_size_allocate (GtkWidget	  *widget,
 				private_data->header_h - 7);
       if (private_data->arrow_win[ARROW_YEAR_LEFT])
 	gdk_window_move_resize (private_data->arrow_win[ARROW_YEAR_LEFT],
-				(allocation->width - 4
+				(allocation->width - 2 * xthickness
 				 - (3 + 2*private_data->arrow_width 
 				    + private_data->max_year_width)),
 				3,
@@ -1371,32 +1361,31 @@ gtk_calendar_size_allocate (GtkWidget	  *widget,
 				private_data->header_h - 7);
       if (private_data->arrow_win[ARROW_YEAR_RIGHT])
 	gdk_window_move_resize (private_data->arrow_win[ARROW_YEAR_RIGHT],
-				(allocation->width - 4 
+				(allocation->width - 2 * xthickness 
 				 - 3 - private_data->arrow_width), 
 				3,
 				private_data->arrow_width,
 				private_data->header_h - 7);
       if (private_data->day_name_win)
 	gdk_window_move_resize (private_data->day_name_win,
-				widget->style->xthickness + INNER_BORDER,
+				xthickness + INNER_BORDER,
 				private_data->header_h + (widget->style->ythickness + INNER_BORDER),
-				allocation->width - (widget->style->xthickness + INNER_BORDER) * 2,
+				allocation->width - (xthickness + INNER_BORDER) * 2,
 				private_data->day_name_h);
       if (private_data->week_win)
 	gdk_window_move_resize (private_data->week_win,
-				(widget->style->xthickness + INNER_BORDER),
+				(xthickness + INNER_BORDER),
 				private_data->header_h + private_data->day_name_h
 				+ (widget->style->ythickness + INNER_BORDER),
 				private_data->week_width,
 				private_data->main_h);
       gdk_window_move_resize (private_data->main_win,
-			      (private_data->week_width ? private_data->week_width + CALENDAR_XSEP :0) 
-			      + (widget->style->xthickness + INNER_BORDER),
+			      private_data->week_width + (xthickness + INNER_BORDER),
 			      private_data->header_h + private_data->day_name_h
 			      + (widget->style->ythickness + INNER_BORDER),
 			      allocation->width 
-			      - (private_data->week_width ? private_data->week_width + CALENDAR_XSEP :0) 
-			      - (widget->style->xthickness + INNER_BORDER) * 2,
+			      - private_data->week_width 
+			      - (xthickness + INNER_BORDER) * 2,
 			      private_data->main_h);
     }
 }
@@ -1408,10 +1397,6 @@ gtk_calendar_expose (GtkWidget	    *widget,
   GtkCalendar *calendar;
   GtkCalendarPrivateData *private_data;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_CALENDAR (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-  
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
   
@@ -1467,7 +1452,7 @@ gtk_calendar_paint_header (GtkWidget *widget)
   /* Clear window */
   gdk_window_clear (private_data->header_win);
   
-  header_width = widget->allocation.width - 4;
+  header_width = widget->allocation.width - 2 * widget->style->xthickness;
   cal_height = widget->allocation.height;
   
   max_month_width = private_data->max_month_width;
@@ -1534,12 +1519,18 @@ gtk_calendar_paint_day_names (GtkWidget *widget)
   PangoLayout *layout;
   PangoRectangle logical_rect;
   GtkCalendarPrivateData *private_data;
-
+  gint focus_padding;
+  gint focus_width;
+  
   g_return_if_fail (GTK_IS_CALENDAR (widget));
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
   gc = calendar->gc;
   
+  gtk_widget_style_get (GTK_WIDGET (widget),
+			"focus-line-width", &focus_width,
+			"focus-padding", &focus_padding,
+			NULL);
   /*
    * Handle freeze/thaw functionality
    */
@@ -1600,11 +1591,12 @@ gtk_calendar_paint_day_names (GtkWidget *widget)
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
       gdk_draw_layout (private_data->day_name_win, gc, 
-		       ((private_data->week_width ? CALENDAR_XSEP : CALENDAR_MARGIN)
-			+ day_wid_sep * i
+		       (CALENDAR_MARGIN +
 			+ private_data->week_width
+			+ (private_data->week_width ? CALENDAR_XSEP : 0)
+			+ day_wid_sep * i
 			+ (day_width - logical_rect.width)/2),
-		       CALENDAR_MARGIN + DAY_YPAD + logical_rect.y,
+		       CALENDAR_MARGIN + focus_width + focus_padding + logical_rect.y,
 		       layout);
     }
   
@@ -1623,6 +1615,8 @@ gtk_calendar_paint_week_numbers (GtkWidget *widget)
   GtkCalendarPrivateData *private_data;
   PangoLayout *layout;
   PangoRectangle logical_rect;
+  gint focus_padding;
+  gint focus_width;
   
   g_return_if_fail (GTK_IS_CALENDAR (widget));
   g_return_if_fail (widget->window != NULL);
@@ -1641,6 +1635,11 @@ gtk_calendar_paint_week_numbers (GtkWidget *widget)
     }
   private_data->dirty_week = 0;
   
+  gtk_widget_style_get (GTK_WIDGET (widget),
+			"focus-line-width", &focus_width,
+			"focus-padding", &focus_padding,
+			NULL);
+  
   /*
    * Clear the window
    */
@@ -1656,13 +1655,13 @@ gtk_calendar_paint_week_numbers (GtkWidget *widget)
     gdk_draw_rectangle (private_data->week_win, gc, TRUE,
 			CALENDAR_MARGIN,
 			0,
-			private_data->week_width - CALENDAR_MARGIN - CALENDAR_XSEP,
+			private_data->week_width - CALENDAR_MARGIN,
 			private_data->main_h - CALENDAR_MARGIN);
   else
     gdk_draw_rectangle (private_data->week_win, gc, TRUE,
 			CALENDAR_MARGIN,
 			CALENDAR_MARGIN,
-			private_data->week_width - CALENDAR_MARGIN - CALENDAR_XSEP,
+			private_data->week_width - CALENDAR_MARGIN,
 			private_data->main_h - 2 * CALENDAR_MARGIN);
   
   /*
@@ -1691,7 +1690,7 @@ gtk_calendar_paint_week_numbers (GtkWidget *widget)
 
       x_loc = (private_data->week_width
 	       - logical_rect.width
-	       - CALENDAR_XSEP - DAY_XPAD);
+	       - CALENDAR_XSEP - focus_padding - focus_width);
 
       gdk_draw_layout (private_data->week_win, gc, x_loc, y_loc, layout);
     }
@@ -1742,6 +1741,8 @@ gtk_calendar_paint_day (GtkWidget *widget,
   gint y_top;
   gint y_loc;
   gint day_xspace;
+  gint focus_width;
+
   GtkCalendarPrivateData *private_data;
   PangoLayout *layout;
   PangoRectangle logical_rect;
@@ -1762,6 +1763,8 @@ gtk_calendar_paint_day (GtkWidget *widget,
       return;
     }
   
+  gtk_widget_style_get (widget, "focus-line-width", &focus_width, NULL);
+
   day_height = row_height (calendar);
   
   day_xspace = private_data->day_width - private_data->max_day_char_width*2;
@@ -1832,9 +1835,14 @@ gtk_calendar_paint_day (GtkWidget *widget,
   if (GTK_WIDGET_HAS_FOCUS (calendar) 
       && calendar->focus_row == row && calendar->focus_col == col)
     {
-      gdk_draw_rectangle (private_data->main_win, calendar->xor_gc, 
-			  FALSE, x_left, y_top,
-			  private_data->day_width-1, day_height-1);
+      gtk_paint_focus (widget->style, 
+		       private_data->main_win,
+		       (calendar->selected_day == day) 
+		          ? GTK_STATE_SELECTED : GTK_STATE_NORMAL, 
+		       NULL, widget, "calendar-day",
+		       x_left, y_top, 
+		       private_data->day_width, 
+		       day_height);
     }
 
   g_object_unref (G_OBJECT (layout));
@@ -2086,7 +2094,6 @@ gtk_calendar_select_month (GtkCalendar *calendar,
 			   guint	month,
 			   guint	year)
 {
-  g_return_val_if_fail (calendar != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_CALENDAR (calendar), FALSE);
   g_return_val_if_fail (month <= 11, FALSE);
   
@@ -2158,7 +2165,6 @@ gboolean
 gtk_calendar_mark_day (GtkCalendar *calendar,
 		       guint	    day)
 {
-  g_return_val_if_fail (calendar != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_CALENDAR (calendar), FALSE);
   
   if (day >= 1 && day <= 31 && calendar->marked_date[day-1] == FALSE)
@@ -2178,7 +2184,6 @@ gboolean
 gtk_calendar_unmark_day (GtkCalendar *calendar,
 			 guint	      day)
 {
-  g_return_val_if_fail (calendar != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_CALENDAR (calendar), FALSE);
   
   if (day >= 1 && day <= 31 && calendar->marked_date[day-1] == TRUE)
@@ -2222,10 +2227,6 @@ gtk_calendar_button_press (GtkWidget	  *widget,
   gint x, y;
   void (* action_func) (GtkCalendar *);
   
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_CALENDAR (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-  
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
   
@@ -2302,9 +2303,6 @@ gtk_calendar_enter_notify (GtkWidget	    *widget,
   GtkCalendar *calendar;
   GtkCalendarPrivateData *private_data;
   
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-  
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
   
@@ -2344,9 +2342,6 @@ gtk_calendar_leave_notify (GtkWidget	    *widget,
   gint row;
   gint col;
   
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-  
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
   
@@ -2398,8 +2393,6 @@ gtk_calendar_paint_arrow (GtkWidget *widget,
   gint state;
   gint width, height;
   
-  g_return_if_fail (widget != NULL);
-  
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
 
@@ -2476,8 +2469,6 @@ gtk_calendar_set_background (GtkWidget *widget)
   GtkCalendarPrivateData *private_data;
   gint i;
   
-  g_return_if_fail (GTK_IS_CALENDAR (widget));
-  
   calendar = GTK_CALENDAR (widget);
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
 
@@ -2511,8 +2502,6 @@ static void
 gtk_calendar_style_set (GtkWidget *widget,
 			GtkStyle  *previous_style)
 {
-  g_return_if_fail (GTK_IS_CALENDAR (widget));
-  
   if (previous_style && GTK_WIDGET_REALIZED (widget))
     gtk_calendar_set_background(widget);
 }
@@ -2521,19 +2510,19 @@ static void
 gtk_calendar_state_changed (GtkWidget	   *widget,
 			    GtkStateType    previous_state)
 {
-  g_return_if_fail (GTK_IS_CALENDAR (widget));
-  
   gtk_calendar_set_background (widget);
 }
 
 static void
-gtk_calendar_destroy (GtkObject *object)
+gtk_calendar_finalize (GObject *object)
 {
   GtkCalendarPrivateData *private_data;
   private_data = GTK_CALENDAR_PRIVATE_DATA (object);
+  
   g_free (private_data);
-  if (GTK_OBJECT_CLASS (parent_class)->destroy)
-    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+  
+  if (G_OBJECT_CLASS (parent_class)->finalize)
+    (* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 static gboolean
@@ -2545,10 +2534,6 @@ gtk_calendar_key_press (GtkWidget   *widget,
   gint old_focus_row;
   gint old_focus_col;
   gint row, col, day;
-
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_CALENDAR (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
   
   calendar = GTK_CALENDAR (widget);
   return_val = FALSE;
