@@ -24,6 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#define GTK_MENU_INTERNALS
+
 #include "gdk/gdkkeysyms.h"
 #include "gtkbindings.h"
 #include "gtkmain.h"
@@ -31,11 +33,11 @@
 #include "gtkmenuitem.h"
 #include "gtkmenushell.h"
 #include "gtksignal.h"
+#include "gtktearoffmenuitem.h"
 #include "gtkwindow.h"
 
 #define MENU_SHELL_TIMEOUT   500
 
-
 enum {
   DEACTIVATE,
   SELECTION_DONE,
@@ -802,6 +804,8 @@ gtk_menu_shell_select_item (GtkMenuShell *menu_shell,
     class->select_item (menu_shell, menu_item);
 }
 
+void _gtk_menu_item_set_placement (GtkMenuItem         *menu_item,
+				   GtkSubmenuPlacement  placement);
 
 static void
 gtk_menu_shell_real_select_item (GtkMenuShell *menu_shell,
@@ -923,6 +927,38 @@ gtk_menu_shell_move_selected (GtkMenuShell  *menu_shell,
     }
 }
 
+/**
+ * _gtk_menu_shell_select_first:
+ * @menu_shell: a #GtkMenuShell
+ * 
+ * Select the first visible child of the menu shell, unless
+ * it's a tearoff item.
+ **/
+void
+_gtk_menu_shell_select_first (GtkMenuShell *menu_shell)
+{
+  GtkWidget *to_select = NULL;
+  GList *tmp_list;
+
+  tmp_list = menu_shell->children;
+  while (tmp_list)
+    {
+      GtkWidget *child = tmp_list->data;
+      
+      if (GTK_WIDGET_VISIBLE (child))
+	{
+	  to_select = child;
+	  if (!GTK_IS_TEAROFF_MENU_ITEM (child))
+	    break;
+	}
+      
+      tmp_list = tmp_list->next;
+    }
+
+  if (to_select)
+    gtk_menu_shell_select_item (menu_shell, to_select);
+}
+
 static void
 gtk_menu_shell_select_submenu_first (GtkMenuShell     *menu_shell)
 {
@@ -931,11 +967,7 @@ gtk_menu_shell_select_submenu_first (GtkMenuShell     *menu_shell)
   menu_item = GTK_MENU_ITEM (menu_shell->active_menu_item); 
   
   if (menu_item->submenu)
-    {
-      GtkMenuShell *submenu = GTK_MENU_SHELL (menu_item->submenu); 
-      if (submenu->children)
-	gtk_menu_shell_select_item (submenu, submenu->children->data); 
-    }
+    _gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_item->submenu));
 }
 
 static void
@@ -971,9 +1003,7 @@ gtk_real_menu_shell_move_current (GtkMenuShell      *menu_shell,
 	  _gtk_menu_item_is_selectable (menu_shell->active_menu_item) &&
 	  GTK_MENU_ITEM (menu_shell->active_menu_item)->submenu)
 	{
-	  menu_shell = GTK_MENU_SHELL (GTK_MENU_ITEM (menu_shell->active_menu_item)->submenu);
-	  if (menu_shell->children)
-	    gtk_menu_shell_select_item (menu_shell, menu_shell->children->data);
+	  gtk_menu_shell_select_submenu_first (menu_shell);
 	}
       else
 	{
@@ -981,8 +1011,15 @@ gtk_real_menu_shell_move_current (GtkMenuShell      *menu_shell,
 	  while (parent_menu_shell && 
 		 (GTK_MENU_SHELL_GET_CLASS (parent_menu_shell)->submenu_placement ==
 		  GTK_MENU_SHELL_GET_CLASS (menu_shell)->submenu_placement))
-	    parent_menu_shell = GTK_MENU_SHELL (parent_menu_shell->parent_menu_shell);
-	  
+            {
+              GtkWidget *tmp_widget = parent_menu_shell->parent_menu_shell;
+
+              if (tmp_widget)
+                parent_menu_shell = GTK_MENU_SHELL (tmp_widget);
+              else
+                parent_menu_shell = NULL;
+            }
+
 	  if (parent_menu_shell)
 	    {
 	      gtk_menu_shell_move_selected (parent_menu_shell, 1);
