@@ -84,13 +84,13 @@ struct _GtkTreeViewPrivate
   /* tree information */
   GtkRBTree *tree;
 
-  gint tab_offset;
   GtkRBNode *button_pressed_node;
   GtkRBTree *button_pressed_tree;
 
   GList *children;
   gint width;
   gint height;
+  gint expander_size;
 
   GtkAdjustment *hadjustment;
   GtkAdjustment *vadjustment;
@@ -102,6 +102,10 @@ struct _GtkTreeViewPrivate
   GtkTreeViewColumn *drag_column;
 
   /* bin_window offset */
+  GtkTreeRowReference *top_row;
+  gint top_row_dy;
+  /* dy == y pos of top_row + top_row_dy */
+  /* we cache it for simplicity of the code */
   gint dy;
   gint drag_column_x;
 
@@ -275,7 +279,10 @@ void         _gtk_tree_view_child_move_resize         (GtkTreeView       *tree_v
 						       gint               y,
 						       gint               width,
 						       gint               height);
-
+void         _gtk_tree_view_queue_draw_node           (GtkTreeView       *tree_view,
+						       GtkRBTree         *tree,
+						       GtkRBNode         *node,
+						       GdkRectangle      *clip_rect);
 
 void _gtk_tree_view_column_realize_button   (GtkTreeViewColumn *column);
 void _gtk_tree_view_column_unrealize_button (GtkTreeViewColumn *column);
